import fs from 'fs';
import { builtinModules } from 'module';
import { resolve, dirname, relative } from 'path';
import enhancedResolve from 'enhanced-resolve';

const create = enhancedResolve.create;
const index = ({ types: t }, { ignoreBuiltins, ignoreBareImport, ignore = [], baseUrl = ".", paths = {}, pathRegexps: optionPathRegexps = {}, relative: optionRelative = true, cache = false, ...resolveOptions }) => {
    // /^\0/ rollup helper
    const ignores = [/^\0/, ...ignore];
    if (ignoreBuiltins) {
        ignores.push(...builtinModules);
    }
    if (ignoreBareImport) {
        ignores.push(/^@?[A-Za-z]/);
    }
    const basePath = resolve(baseUrl);
    const pathEntries = Object.entries(paths).map(([prefix, path]) => [prefix, path]);
    const pathRegexps = Object.entries(optionPathRegexps).map(([regexp, path]) => [
        new RegExp(regexp),
        path,
    ]);
    if (!cache) {
        resolveOptions.fileSystem = fs;
    }
    resolveOptions = {
        conditionNames: ["import"],
        extensions: [".tsx", ".ts", ".jsx", ".mjs", ".js", ".cjs", ".json", ".node"],
        ...resolveOptions,
    };
    const enhancedResolve = create.sync(resolveOptions);
    const resolve$1 = (path, request, deps) => {
        if (ignores.some((ignore) => (typeof ignore === "string" ? ignore === request : ignore.test(request)))) {
            deps.push(request);
            return request;
        }
        for (const [prefix, path] of pathEntries) {
            if (request.startsWith(prefix)) {
                request = resolve(basePath, path + request.slice(prefix.length));
                break;
            }
        }
        for (const [regexp, path] of pathRegexps) {
            if (regexp.test(request)) {
                request = resolve(basePath, request.replace(regexp, path));
                break;
            }
        }
        const dir = dirname(path);
        request = enhancedResolve(dir, request) || request;
        deps.push(request);
        if (optionRelative) {
            request = relative(dir, request);
            if (request[0] !== "/" && request[0] !== ".") {
                request = "./" + request;
            }
        }
        return request;
    };
    const transformDeclaration = (path, state) => {
        const { source } = path.node;
        if (!state.filename || !source) {
            return;
        }
        let modulePath = source.value;
        try {
            modulePath = resolve$1(state.filename, modulePath, state.file.metadata.deps);
        }
        catch (e) {
            console.log(path.buildCodeFrameError(e?.message));
            return;
        }
        const sourcePath = path.get("source");
        if (Array.isArray(sourcePath)) {
            return;
        }
        sourcePath.replaceWith(t.stringLiteral(modulePath));
    };
    const transformImport = (path, state) => {
        if (!state.filename || !path.parentPath.isCallExpression()) {
            return;
        }
        const arg = path.parentPath.get("arguments")[0];
        let modulePath;
        if (arg.isTemplateLiteral() && arg.node.expressions.length === 0) {
            modulePath = arg.node.quasis[0].value.cooked;
        }
        if (arg.isStringLiteral()) {
            modulePath = arg.node.value;
        }
        if (!modulePath) {
            return;
        }
        try {
            modulePath = resolve$1(state.filename, modulePath, state.file.metadata.deps);
        }
        catch (e) {
            console.log(path.buildCodeFrameError(e?.message));
            return;
        }
        arg.replaceWith(t.stringLiteral(modulePath));
    };
    return {
        name: "resolve",
        pre(_state) {
            this.file.metadata.deps = [];
        },
        post() {
            this.file.metadata.deps = [...new Set(this.file.metadata.deps)];
        },
        visitor: {
            ImportDeclaration: transformDeclaration,
            ExportAllDeclaration: transformDeclaration,
            ExportNamedDeclaration: transformDeclaration,
            Import: transformImport,
        },
    };
};

export { index as default };
//# sourceMappingURL=index.js.map
